@@ -172,7 +172,6 @@ FolderModel::FolderModel(QObject *parent)
     , m_screenUsed(false)
     , m_screenMapper(ScreenMapper::instance())
     , m_complete(false)
-    , watcher(nullptr)
     , m_currentActivity(KActivities::Consumer().currentActivity())
     , m_showHiddenFiles(false)
 {
@@ -368,6 +367,17 @@ void FolderModel::newFileMenuItemRejected(const QUrl &url)
     setCreatingNewItems(false);
 }
 
+void FolderModel::setUnsortedModeOnDrop()
+{
+    // The positioner will override old icon layout by
+    // sorted icon layout after drag and drop operation.
+    if (m_sortMode != -1) {
+        m_unsortedModeOnDrop = true;
+        setSortMode(-1);
+        m_unsortedModeOnDrop = false;
+    }
+}
+
 QString FolderModel::url() const
 {
     return m_url;
@@ -407,16 +417,9 @@ void FolderModel::setUrl(const QString &url)
         m_dirWatch = new KDirWatch(this);
         connect(m_dirWatch, &KDirWatch::created, this, &FolderModel::iconNameChanged);
         connect(m_dirWatch, &KDirWatch::dirty, this, &FolderModel::iconNameChanged);
-        m_dirWatch->addFile(resolvedNewUrl.toLocalFile() + QStringLiteral("/.directory"));
+        m_dirWatch->addFile(DesktopSchemeHelper::getFileUrl(resolvedNewUrl.toString()).remove(QStringLiteral("file://")) + QStringLiteral("/.directory"));
     }
 
-    if (watcher) {
-        watcher->deleteLater();
-    }
-    watcher = new QFileSystemWatcher(this);
-    addDirectoriesRecursively(resolvedNewUrl.toString(), watcher);
-    connect(watcher, &QFileSystemWatcher::directoryChanged, this, &FolderModel::refresh);
-
     if (dragging()) {
         m_urlChangedWhileDragging = true;
     }
@@ -429,36 +432,6 @@ void FolderModel::setUrl(const QString &url)
     }
 }
 
-void FolderModel::addDirectoriesRecursively(const QString &resolvedNewUrl, QFileSystemWatcher *watcher)
-{
-    QStack<QString> directoryStack;
-    directoryStack.push(resolvedNewUrl);
-
-    while (!directoryStack.isEmpty()) {
-        QString currentDir = directoryStack.pop();
-
-        // Add current directory to watcher
-        watcher->addPath(DesktopSchemeHelper::getFileUrl(currentDir));
-
-        QDir dir(currentDir);
-        dir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
-        QList<QFileInfo> subdirInfoList = dir.entryInfoList();
-
-        // Extract file paths and add them to subdirs list
-        QStringList subdirs;
-        for (const QFileInfo &subdirInfo : subdirInfoList) {
-            if (subdirInfo.isDir()) {
-                subdirs.append(subdirInfo.filePath());
-            }
-        }
-
-        // Push subdirectories onto the stack for further processing
-        for (const QString &subdir : subdirs) {
-            directoryStack.push(subdir);
-        }
-    }
-}
-
 QUrl FolderModel::resolvedUrl() const
 {
     return m_dirModel->dirLister()->url();
@@ -912,8 +885,22 @@ void FolderModel::rename(int row, const QString &name)
     }
 
     QModelIndex idx = index(row, 0);
+    const QString filename = data(idx, UrlRole).toString();
+    Q_EMIT itemAboutToRename(filename);
     m_dirModel->setData(mapToSource(idx), name, Qt::EditRole);
-    connect(m_dirModel, &KDirModel::dataChanged, this, &FolderModel::itemRenamed, Qt::SingleShotConnection);
+    connect(
+        m_dirModel,
+        &KDirModel::dataChanged,
+        this,
+        [=, this](const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles) {
+            Q_UNUSED(roles)
+            QString newFilename;
+            if (topLeft == bottomRight) {
+                newFilename = data(mapFromSource(topLeft), UrlRole).toString();
+            }
+            Q_EMIT itemRenamed(filename, newFilename);
+        },
+        Qt::SingleShotConnection);
 }
 
 int FolderModel::fileExtensionBoundary(int row)
@@ -1180,7 +1167,9 @@ void FolderModel::dragSelectedInternal(int x, int y)
     DragTracker::self()->setDragInProgress(nullptr, false);
     m_urlChangedWhileDragging = false;
 
-    if (m_dirModel->dirLister()->url() == currentUrl) {
+    // If we spring-load into another folder and back (with the back button or draggin outside a popup)
+    // the first condition will be true, but we don't want to do this with an empty dragIndexes
+    if (m_dirModel->dirLister()->url() == currentUrl && !m_dragIndexes.isEmpty()) {
         const QModelIndex first(m_dragIndexes.first());
         const QModelIndex last(m_dragIndexes.last());
         m_dragIndexes.clear();
@@ -1276,7 +1265,7 @@ void FolderModel::drop(QQuickItem *target, QObject *dropEvent, int row, bool sho
             return;
         }
 
-        setSortMode(-1);
+        setUnsortedModeOnDrop();
 
         for (const auto &url : mimeData->urls()) {
             m_dropTargetPositions.insert(url.fileName(), dropPos);
@@ -1300,7 +1289,7 @@ void FolderModel::drop(QQuickItem *target, QObject *dropEvent, int row, bool sho
 
     if (m_usedByContainment && !m_screenMapper->sharedDesktops()) {
         if (isDropBetweenSharedViews(mimeData->urls(), dropTargetFolderUrl)) {
-            setSortMode(-1);
+            setUnsortedModeOnDrop();
             const QList<QUrl> urls = mimeData->urls();
             for (const auto &url : urls) {
                 m_dropTargetPositions.insert(url.fileName(), dropPos);
@@ -2220,11 +2209,10 @@ void FolderModel::setApplet(Plasma::Applet *applet)
                     m_screenMapper->setCorona(corona);
                 }
                 setScreen(containment->screen());
-                connect(containment, &Plasma::Containment::screenGeometryChanged, this, &FolderModel::screenGeometryChanged);
-                connect(containment, &Plasma::Containment::availableRelativeScreenRectChanged, this, &FolderModel::availableRelativeScreenRectChanged);
                 connect(containment, &Plasma::Containment::screenChanged, this, [this](int newScreen) {
                     setScreen(newScreen, SetScreenActions::MoveIcons);
                 });
+                connect(containment, &Plasma::Containment::screenGeometryChanged, this, &FolderModel::screenGeometryChanged);
             }
         }
 
@@ -2232,8 +2220,20 @@ void FolderModel::setApplet(Plasma::Applet *applet)
     }
 }
 
+#ifdef BUILD_TESTING
+void FolderModel::setScreenResolution(const QSizeF &size)
+{
+    m_screenResolution = size;
+}
+#endif
+
 QRectF FolderModel::screenGeometry()
 {
+#ifdef BUILD_TESTING
+    if (m_screenResolution.isValid()) {
+        return QRectF(QPointF(), m_screenResolution);
+    }
+#endif
     if (m_applet) {
         Plasma::Containment *containment = m_applet->containment();
         if (containment) {
@@ -2243,6 +2243,11 @@ QRectF FolderModel::screenGeometry()
     return QRectF();
 }
 
+bool FolderModel::unsortedModeOnDrop()
+{
+    return m_unsortedModeOnDrop;
+}
+
 bool FolderModel::creatingNewItems() const
 {
     return m_creatingNewItems;
